function main()
%% This application shows electric and magnetic field distribution radiated by several sources.
% Source type can be selected as: spherical with and without free space losses,
% and plane waves. Several cuts through solution space can be defined and
% investigated in separate graph. Field can be also animated to see traveling of
% wavefronts during time. Application settings can be saved and loaded for later
% usage.
%
% © 2020, Viktor Adler, CTU in Prague, adlervik@fel.cvut.cz

%% Default state of app
freq = 2e9; % frequency of harmonic field source
% definition of field sources
sourcesData = table(...
   [0; 0], ... % powers in dBm
   [0; 0], ... % phases
   ["[-0.15 0]"; "[0.15 0]"], ... % positions, [X, Y]
   [-10; 10], ... % azimuths of plane waves in deg.
   [true; true], ... % enable
   'VariableNames', {'Power (dBm)', 'Phase (deg.)', 'Position (m)', ...
   'Azimuth (deg.)', 'Enable'});

XMin = -0.5; % min. X coordinate of solution space
XMax = 0.5; % max. X coordinate of solution space
XSamples = 200; % number of samples per X-direction of solution space
YMin = -0.5; % min. Y coordinate of solution space
YMax = -2.5; % max. Y coordinate of solution space
YSamples = 400; % number of samples per Y-direction of solution space
% list of all possible line markers and colors for interactive selection from table
lineMarkers = categorical("none", ...
   ["none", "+", "o", "*", ".", "x", "s", "d", "^", "v", "p", "h"]);
lineColors = categorical("r", ["r", "g", "b", "c", "m", "y", "k"]);
% definition of field slices
slicesData = table(...
   ["[0, 0]"; "[-1, 1]"], ... % X ranges of slices
   ["[-1, -2]"; "[-0.7, -0.7]"], ... % Y ranges of slices
   [101; 201], ... % number of points
   repmat(lineMarkers, 2, 1), ... % markers
   repmat(lineColors, 2, 1), ... % colors of slices
   [true; true], ... % enable
   'VariableNames', {'X (m)', 'Y (m)', 'Points (-)', 'Marker', 'Color', 'Enable'});

% list of variables to save into *.mat file
variableToSaveLoad = {'freq', 'sourcesData', 'XMin', 'XMax', 'XSamples', ...
   'YMin', 'YMax', 'YSamples', 'slicesData'};
maxDBRange = 100; % max. range of shown values in dB range
fieldColormap = @jet; % colormap of field data

%% Constans
c0 = 299792458; % speed of light
mu0 = 4*pi*1e-7; % permeability of vacuum
Z0 = mu0*c0; % impedance of free space
origin = [0, 0]; % origin of coordinate system

%% Allocations
% to utilize sharing of variables via nested functions it is necessary to define
% them inside main function
X = []; % X-coordinates of points in solution space
Y = []; % Y-coordinates of points in solution space
R = []; % range of every point from solution space from origin
fieldInPlane = []; % computed field in all X and Y points
sourcesX = []; % X-coordinates of all sources
sourcesY = []; % Y-coordinates of all sources
sourcePowers = []; % powers of all sources
sourcePhases = []; % phases of all sources
sourceAzimuths = []; % azimuths of all planar wave sources
slicesX = []; % X-coordinates of all slices
slicesY = []; % Y-coordinates of all slices
hSlicesLines = gobjects(0, 1); % allocation of ines representing slices
slicesColors = []; % colors of slices
slicesPoints = []; % number of points representing slice of solution space
slicesMarkers = []; % markers utilized for showing of field slices
sliceID = []; % identifier of slices
hSlicesField = gobjects(0, 1); % allocation of lines representing field along slices
fieldAtSlices = []; % values of field along slices
slicesLength = []; % length of slices
slicesR = []; % range of all slice points from arigin
timeStep = []; % time step during animation of field
fieldCompFcn = []; % handle to function computing a field
RCompFcn = []; % handle to function computing range of points from origin
RInputArg = []; % input arguments for computing range from origin
k = []; % wave number
nEnSlices = []; % number of enabled slices

%% Timer managing field animation
hTimer = timer('ExecutionMode', 'fixedSpacing', 'Period', 0.1);
nStepsPerPeriod = 20; % number of steps of animation per source signal period
time = 0; % default time when field is computed

%% GUI building
figSize = [1500, 800];
screenSize = get(groot, 'ScreenSize');
figPosition = [(screenSize(3:4) - figSize)/2, figSize];

hFig = uifigure('Position', figPosition, 'Name', 'Spherical vs. Plane Wave');
hGridMain = uigridlayout(hFig, 'ColumnWidth', {530, '1x'}, ...
   'RowHeight', {310, 95, 275, 50, '1x'}, 'Scrollable', true);

% Menu
hMenuFile = uimenu(hFig, 'Text', 'File');
hMenuSave = uimenu(hMenuFile, 'Text', 'Save');
hMenuLoad = uimenu(hMenuFile, 'Text', 'Load');

% Sources panel
hPanelSources = uipanel(hGridMain, 'Title', 'Sources');
hGridSources = uigridlayout(hPanelSources, 'Scrollable', true, ...
   'RowHeight', {22, 200, 22}, 'ColumnWidth', {'1x', '1x', '1x', '1x'});
uilabel(hGridSources, 'Text', 'Frequency:');
hEditFrequency = uieditfield(hGridSources, 'numeric', 'Limits', [0, Inf], ...
   'ValueDisplayFormat', '%f GHz', 'LowerLimitInclusive', false, ...
   'UpperLimitInclusive', false);
hTableSources = uitable(hGridSources, 'ColumnEditable', true, ...
   'RowName', 'numbered', 'FontName', 'Courier New', ...
   'ColumnWidth', {'auto', 'auto', 'auto', 'auto', 'fit'}, 'Tooltip', ...
   ["Specify spherical and plane wave sources.", ...
   "Power: power of isotropic source, or power density  per m^2 of plane wave.", ...
   "Phase: phase of wave at zero time in origin.", ...
   "Position: spherical source position in [X, Y] m format.", ...
   "Azimuth: planar wave azimuth.", ...
   "Enable: turn source on or off."]);
hTableSources.Layout.Row = 2;
hTableSources.Layout.Column = [1, 4];
hButtAddSource = uibutton(hGridSources, 'Text', 'Add Source');
hButtAddSource.Layout.Column = 2;
hButtDeleteSource = uibutton(hGridSources, 'Text', 'Delete Source');
hButtDeleteSource.Layout.Column = 3;

% Solution Space Panel
hPanelSolSpace = uipanel(hGridMain, 'Title', 'Solution Space');
hPanelSolSpace.Layout.Row = 2;
hPanelSolSpace.Layout.Column = 1;
hGridSolSpace = uigridlayout(hPanelSolSpace, 'RowHeight', {22, 22}, ...
   'ColumnWidth', {'fit', '1x', 'fit', '1x', 'fit', '1x'});
uilabel(hGridSolSpace, 'Text', 'Xmin:');
solSpaceTooltip = ["Define range of space where a field will be computed.", ...
   "Higher number of points produce smoother results."];
hEditXMin = uieditfield(hGridSolSpace, 'numeric', 'ValueDisplayFormat', '%.3e m', ...
   'LowerLimitInclusive', false, 'UpperLimitInclusive', false, ...
   'Tooltip', solSpaceTooltip);

uilabel(hGridSolSpace, 'Text', 'Xmax:');
hEditXMax = uieditfield(hGridSolSpace, 'numeric', 'ValueDisplayFormat', '%.3e m', ...
   'LowerLimitInclusive', false, 'UpperLimitInclusive', false, ...
   'Tooltip', solSpaceTooltip);

uilabel(hGridSolSpace, 'Text', 'Samples:');
hEditXSamples = uieditfield(hGridSolSpace, 'numeric', 'ValueDisplayFormat', '%u', ...
   'Limits', [2, inf], 'UpperLimitInclusive', false, ...
   'Tooltip', solSpaceTooltip);

uilabel(hGridSolSpace, 'Text', 'Ymin:');
hEditYMin = uieditfield(hGridSolSpace, 'numeric', 'ValueDisplayFormat', '%.3e m', ...
   'LowerLimitInclusive', false, 'UpperLimitInclusive', false, ...
   'Tooltip', solSpaceTooltip);

uilabel(hGridSolSpace, 'Text', 'Ymax:');
hEditYMax = uieditfield(hGridSolSpace, 'numeric', 'ValueDisplayFormat', '%.3e m', ...
   'LowerLimitInclusive', false, 'UpperLimitInclusive', false, ...
   'Tooltip', solSpaceTooltip);

uilabel(hGridSolSpace, 'Text', 'Samples:');
hEditYSamples = uieditfield(hGridSolSpace, 'numeric', 'ValueDisplayFormat', '%u', ...
   'Limits', [2, inf], 'UpperLimitInclusive', false, ...
   'Tooltip', solSpaceTooltip);

% Slices panel
hPanelSlices = uipanel(hGridMain, 'Title', 'Slices');
hPanelSlices.Layout.Row = 3;
hPanelSlices.Layout.Column = 1;

hGridSlices = uigridlayout(hPanelSlices, 'RowHeight', {200, 22}, ...
   'ColumnWidth', {'1x', '1x', '1x', '1x'}, 'Scrollable', true);
hTableSlices = uitable(hGridSlices, 'ColumnEditable', true, ...
   'RowName', 'numbered', 'FontName', 'Courier New', ...
   'ColumnWidth', {'auto', 'auto', 'fit', 'fit', 'fit', 'fit'}, 'Tooltip', ...
   ["Definition of field slices.", ...
   "X, Y: coordinates of end points of slices.", ...
   "Points: number of points representing field values along slice.", ...
   "Marker: style of line marker, double-click for selection.", ...
   "Color: color of line, double-click for selection.", ...
   "Enable: turn slice on or off."]);
hTableSlices.Layout.Column = [1, 4];
hButtAddSlice = uibutton(hGridSlices, 'Text', 'Add Slice');
hButtAddSlice.Layout.Column = 2;
hButtDeleteSlice = uibutton(hGridSlices, 'Text', 'Delete Slice');
hButtDeleteSlice.Layout.Column = 3;

% Solve panel
hPanelSolve = uipanel(hGridMain); % panel contains just a Solve! button
hPanelSolve.Layout.Row = 4;
hPanelSolve.Layout.Column = 1;
hGridSolve = uigridlayout(hPanelSolve, 'ColumnWidth', {'1x', '1x', '1x'}, ...
   'RowHeight', {'1x'});
hButtSolve = uibutton(hGridSolve, 'Text', 'Solve!');
hButtSolve.Layout.Column = 2;

% Results panel
hPanelResults = uipanel(hGridMain, 'Title', 'Results');
hPanelResults.Layout.Row = [1, 5];
hPanelResults.Layout.Column = 2;
hGridResults = uigridlayout(hPanelResults, 'RowHeight', {'1x'}, ...
   'ColumnWidth', {'1x', '1x'});
hGridField = uigridlayout(hGridResults, 'RowHeight', {22, 22, 40, '1x'}, ...
   'ColumnWidth', {'1x', 120, 30});

% radio buttons
% in UserData property are saved identifiers indicating which button is selected
hButtGroupWaveType = uibuttongroup(hGridField);
uiradiobutton(hButtGroupWaveType, 'Text', 'Spherical+Loss', ...
   'Position', [5, 0, 100, 22], 'UserData', "spherical+loss");
uiradiobutton(hButtGroupWaveType, 'Text', 'Spherical', ...
   'Position', [120, 0, 70, 22], 'UserData', "spherical");
uiradiobutton(hButtGroupWaveType, 'Text', 'Plane', ...
   'Position', [200, 0, 60, 22], 'UserData', "plane");

hButtGroupEH = uibuttongroup(hGridField);
uiradiobutton(hButtGroupEH, 'Text', 'E', ...
   'Position', [5, 0, 40, 22], 'UserData', "E");
uiradiobutton(hButtGroupEH, 'Text', 'H', ...
   'Position', [50, 0, 40, 22], 'UserData', "H");

% in UserData property are saved handle function which are directly utilized for
% processing of field data
hButtGroupComplex = uibuttongroup(hGridField);
hButtGroupComplex.Layout.Row = 2;
hButtGroupComplex.Layout.Column = 1;
uiradiobutton(hButtGroupComplex, 'Text', 'Abs.', ...
   'Position', [5, 0, 45, 22], 'UserData', @abs);
uiradiobutton(hButtGroupComplex, 'Text', 'Real.', ...
   'Position', [55, 0, 45, 22], 'UserData', @real);
uiradiobutton(hButtGroupComplex, 'Text', 'Imag.', ...
   'Position', [105 0, 50, 22], 'UserData', @imag);
uiradiobutton(hButtGroupComplex, 'Text', 'Phase', ...
   'Position', [160, 0, 55, 22], 'UserData', @(x)angle(x)/pi*180);

hButtGroupLinLog = uibuttongroup(hGridField);
uiradiobutton(hButtGroupLinLog, 'Text', 'Lin.', ...
   'Position', [5, 0, 40, 22], 'UserData', @(x)x());
uiradiobutton(hButtGroupLinLog, 'Text', 'dBMax', ...
   'Position', [50, 0, 65, 22], 'UserData', @(x)logScale(x));

allRadioButt = [hButtGroupComplex.Children(:); hButtGroupWaveType.Children(:);
   hButtGroupEH.Children(:); hButtGroupLinLog.Children(:)];

% Play, stop buttons
hButtPlay = uibutton(hGridField, 'Text', '', 'Icon', getPlayIcon(), ...
   'IconAlignment', 'center');
hButtPlay.Layout.Row = 1;
hButtPlay.Layout.Column = 3;

hButtStop = uibutton(hGridField, 'Text', '', 'Icon', zeros(16, 16, 3), ...
   'IconAlignment', 'center');
hButtStop.Layout.Row = 2;
hButtStop.Layout.Column = 3;

% dB range slider
hGridDBRangeLim = uigridlayout(hGridField, 'ColumnWidth', {65, '1x'}, ...
   'RowHeight', {'1x'}, 'Padding', zeros(1, 4), 'ColumnSpacing', 0);
hGridDBRangeLim.Layout.Column = [1, 3];
uilabel(hGridDBRangeLim, 'Text', 'dB Range:');
hSliderDBRange = uislider(hGridDBRangeLim, 'Limits', [0, maxDBRange], ...
   'Value', maxDBRange, 'Tooltip', ["Set dynamic range of data shown in decibels.", ...
   "The maximal shown value has always 0 dB."]);

% Field axes
hAxField = uiaxes(hGridField, 'DataAspectRatio', [1, 1, 1], ...
   'Colormap', fieldColormap(), 'Interactions', [panInteraction, zoomInteraction]);
hAxField.Layout.Row = 4;
hAxField.Layout.Column = [1, 2];
hAxField.XAxis.Label.String = 'X (m)';
hAxField.YAxis.Label.String = 'Y (m)';
axtoolbar(hAxField, {'pan', 'zoomin', 'zoomout', 'restoreview'});
hSurf = surface('Parent', hAxField, 'XData', [0, 1], 'YData', [0, 1], ...
   'ZData', zeros(2), 'CData', nan(2), 'EdgeColor', 'none');
colorbar(hAxField);

% Grid with graph showing slices of field
hGridSlices = uigridlayout(hGridResults, 'RowHeight', {22, '1x'}, ...
   'ColumnWidth', {'fit', 'fit', '1x'});
uilabel(hGridSlices, 'Text', 'Selected slice:');
hDropDownSlicePick = uidropdown(hGridSlices, 'Items', {}, 'ItemsData', []);
hAxSlices = uiaxes(hGridSlices, 'XGrid', true, 'YGrid', true, ...
   'XMinorGrid', true, 'YMinorGrid', true);
hAxSlices.Layout.Row = 2;
hAxSlices.Layout.Column = [1, 3];
hAxSlices.XAxis.Label.String = 'Slice Length (m)';
hAxSlices.YAxis.TickLabelFormat = '%.2f';

%% Calbacks
hButtAddSource.ButtonPushedFcn = ...
   @(a, b)addTableRow(hTableSources, {0, 0, "[0 0]", 0, true});
hButtAddSlice.ButtonPushedFcn = ...
   @(a, b)addTableRow(hTableSlices, {"[0 1]", "[0 1]", 101, lineMarkers, lineColors, true});
hButtDeleteSource.ButtonPushedFcn = @(a, b)deleteTableRow(hTableSources);
hButtDeleteSlice.ButtonPushedFcn = @(a, b)deleteTableRow(hTableSlices);
hTableSources.CellSelectionCallback = @saveTableSelectedRow;
hTableSlices.CellSelectionCallback = @saveTableSelectedRow;
hDropDownSlicePick.ValueChangedFcn = @(a, b)selectSlice();
% create function handle which will be called when format of data is changed
% it consists of function handles saved in UserData property of individual radio
% buttons
resultStyleFcn = ...
   @(a, b)updateGraphs(hButtGroupComplex.SelectedObject.UserData, ...
   hButtGroupLinLog.SelectedObject.UserData, ...
   hButtGroupEH.SelectedObject.UserData);
hButtGroupComplex.SelectionChangedFcn = resultStyleFcn;
hButtGroupLinLog.SelectionChangedFcn = resultStyleFcn;
% create function handle which will be called when source type or E/H is changed
solveFieldTypeFcn = @(a, b)solveFromButton(hButtGroupWaveType.SelectedObject.UserData, ...
   hButtGroupEH.SelectedObject.UserData);
hButtGroupEH.SelectionChangedFcn = solveFieldTypeFcn;
hButtGroupWaveType.SelectionChangedFcn = solveFieldTypeFcn;
hButtSolve.ButtonPushedFcn = solveFieldTypeFcn;

hSliderDBRange.ValueChangedFcn = @(src, event)changeDynamicRange(event);
hButtPlay.ButtonPushedFcn = @(a, b)startTimer();
hButtStop.ButtonPushedFcn = @(a, b)stopTimer();

hFig.CloseRequestFcn = @(a, b)closeFigure(hFig, hTimer);
% function periodically called by the timer
hTimer.TimerFcn = @(a, b)solveFromPlay();

hMenuSave.MenuSelectedFcn = @(a, b)saveSettings();
hMenuLoad.MenuSelectedFcn = @(a, b)loadSettings();

%% Set default values to GUI
setupGUI();
solveFieldTypeFcn();

   function setupGUI()
      % setup of GUI right after creating it, or after loading settings
      hEditFrequency.Value = freq/1e9;
      hTableSources.Data = sourcesData;
      hEditXMin.Value = XMin;
      hEditXMax.Value = XMax;
      hEditXSamples.Value = XSamples;
      hEditYMin.Value = YMin;
      hEditYMax.Value = YMax;
      hEditYSamples.Value = YSamples;
      hTableSlices.Data = slicesData;
      set(allRadioButt, 'Enable', false);
      hSliderDBRange.Enable = false;
      set([hButtPlay, hButtStop], 'Enable', false);
   end

   function saveSettings()
      % save settings of application
      hFig.Visible = false;
      [fileName, pathName] = uiputfile('*.mat', 'Save Settings');
      if isequal(fileName, 0)
         hFig.Visible = true;
         return;
      end
      filePath = fullfile(pathName, fileName);
      save(filePath, variableToSaveLoad{:});
      hFig.Visible = true;
   end

   function loadSettings()
      % load settings of application
      hFig.Visible = false;
      [fileName, pathName] = uigetfile('*.mat', 'Load Settings');
      if isequal(fileName, 0)
         hFig.Visible = true;
         return;
      end
      filePath = fullfile(pathName, fileName);
      load(filePath, variableToSaveLoad{:});
      setupGUI();
      hFig.Visible = true;
   end

   function startTimer()
      hButtPlay.Enable = false;
      hButtSolve.Enable = false;
      set(hButtGroupEH.Children, 'Enable', false); % disable change E/H
      set(hButtGroupWaveType.Children, 'Enable', false); % disable source type change
      start(hTimer); % start timer which will periodicaly update results
   end

   function stopTimer()
      hButtPlay.Enable = true;
      hButtSolve.Enable = true;
      set(hButtGroupEH.Children, 'Enable', true); % enable change E/H
      set(hButtGroupWaveType.Children, 'Enable', true); % enable source type change
      stop(hTimer);
      time = 0; % set default zero time
      solveField(); % compute field values for zero time
      resultStyleFcn(); % update graphs
   end

   function readRadioButtons(sphericalOrPlane, EOrH)
      % check state of all radio buttons and properly set computation functions
      sphereRInputArg = {sourcesX, sourcesY}; % coordinates of spherical sources
      planeRInputArg = {origin, sourceAzimuths, k};
      
      if EOrH == "E"
         if sphericalOrPlane == "spherical"
            fieldCompFcn = @computeRadialE;
            RCompFcn = @computeRadialR;
            RInputArg = sphereRInputArg;
         elseif sphericalOrPlane == "spherical+loss"
            fieldCompFcn = @computeLossyRadialE;
            RCompFcn = @computeRadialR;
            RInputArg = sphereRInputArg;
         elseif sphericalOrPlane == "plane"
            fieldCompFcn = @computePlaneE;
            RCompFcn = @computePlaneR;
            RInputArg = planeRInputArg;
         end
      elseif EOrH == "H"
         if sphericalOrPlane == "spherical"
            fieldCompFcn = @computeRadialH;
            RCompFcn = @computeRadialR;
            RInputArg = sphereRInputArg;
         elseif sphericalOrPlane == "spherical+loss"
            fieldCompFcn = @computeLossyRadialH;
            RCompFcn = @computeRadialR;
            RInputArg = sphereRInputArg;
         elseif sphericalOrPlane == "plane"
            fieldCompFcn = @computePlaneH;
            RCompFcn = @computePlaneR;
            RInputArg = planeRInputArg;
         end
      end
   end

   function solveFromButton(sphericalOrPlane, EOrH)
      % whole computation of field invoked from Solve! button and E/H and type
      % of source radio buttons
      freq = hEditFrequency.Value*1e9; % convert frequency from GHz to Hz
      timeStep = 1/freq/nStepsPerPeriod; % compute time step of animation
      lambda = c0/freq; % wavelength
      k = 2*pi/lambda; % wave number
      errorMessage = readSources(); % read settings of sources
      if ~isempty(errorMessage)
         uialert(hFig, errorMessage, 'Invalid Source Definition');
         return;
      end
      errorMessage = readSolSpace(); % read settings of solution space
      if ~isempty(errorMessage)
         uialert(hFig, errorMessage, 'Invalid Solution Space Definition');
         return;
      end
      errorMessage = readSlices(); % read settings of slices
      if ~isempty(errorMessage)
         uialert(hFig, errorMessage, 'Invalid Slices Definition');
         return;
      end
      % check state of all radio buttons and set appropriate function handles
      readRadioButtons(sphericalOrPlane, EOrH);
      solveR(); % compute distances of all necessary points from origin
      solveField(); % compute field around the sources
      % enable radio buttons after first successful computing of field
      set(allRadioButt, 'Enable', true);
      % enable play and stop buttons
      set([hButtPlay, hButtStop], 'Enable', true);
      % update graphs with appropriate settings
      resultStyleFcn();
   end

   function solveFromPlay()
      % periodicaly called by the timer
      time = time + timeStep;
      solveField(); % compute field in new time
      resultStyleFcn(); % update graphs
   end

   function solveField()
      % compute field using appropriate function according to selected field
      % source
      fieldInPlane = fieldCompFcn(sourcePowers, sourcePhases, ...
         R, Z0, k, time, freq);
      % compute also field along all slices using same functions
      fieldAtSlices = cell(nEnSlices, 1);
      for iSlice = 1:nEnSlices
         fieldAtSlices{iSlice} = fieldCompFcn(sourcePowers, ...
            sourcePhases, slicesR{iSlice}, Z0, k, time, freq);
      end
   end

   function solveR()
      % compute distances of points from solution space
      R = RCompFcn(RInputArg{:}, X, Y);
      % compute distances of points on slices
      slicesR = cell(nEnSlices, 1);
      for iSlice = 1:nEnSlices
         slicesR{iSlice} = RCompFcn(RInputArg{:}, slicesX{iSlice}, slicesY{iSlice});
      end
   end

   function errorMessage = readSlices()
      % read data of slices and check values
      errorMessage = '';
      slicesData = hTableSlices.Data;
      allSliceEnable = slicesData.Enable;
      nEnSlices = sum(allSliceEnable);
      % read and check data of slices if
      if nEnSlices
         sliceID = find(allSliceEnable); % ID of slices as number of rows in table
         % check number of points of slices
         slicesPoints = mat2cell(slicesData.("Points (-)")(allSliceEnable), ones(nEnSlices, 1));
         try
            cellfun(@(x)validateattributes(x, {'numeric'}, ...
               {'scalar', 'finite', 'integer', '>=', 2}), ...
               slicesPoints);
         catch ME
            errorMessage = ['Invalid number of slice points. ', ME.message];
            return;
         end
         % read and check X coordinates of end points
         slicesX = arrayfun(@str2num, ...
            slicesData.("X (m)")(allSliceEnable), 'UniformOutput', false);
         try
            cellfun(@(x)validateattributes(x, {'numeric'}, ...
               {'size', [1, 2], 'finite'}), slicesX);
         catch ME
            errorMessage = ['Invalid X-coordinate of end points of slice. ', ME.message];
            return;
         end
         slicesX = cellfun(@(x, y)linspace(x(1), x(2), y).', slicesX, slicesPoints, ...
            'UniformOutput', false);
         % read and check Y coordinates of end points
         slicesY = arrayfun(@str2num, ...
            slicesData.("Y (m)")(allSliceEnable), 'UniformOutput', false);
         try
            cellfun(@(x)validateattributes(x, {'numeric'}, ...
               {'size', [1, 2], 'finite'}), slicesY);
         catch ME
            errorMessage = ['Invalid Y-coordinate of end points of slice. ', ME.message];
            return;
         end
         slicesY = cellfun(@(x, y)linspace(x(1), x(2), y).', slicesY, slicesPoints, ...
            'UniformOutput', false);
         slicesLength = cellfun(@(x, y)hypot((x - x(1)), y - y(1)) - ...
            norm([x(end) - x(1), y(end) - y(1)])/2, ...
            slicesX, slicesY, 'UniformOutput', false);
         % check colors of slices
         slicesColors = string(slicesData.Color(allSliceEnable));
         areColorsKnown = all(ismember(slicesColors, string(categories(lineColors))));
         if ~areColorsKnown
            errorMessage = 'Unknown line colors.';
            return;
         end
         % check markers of slices
         slicesMarkers = string(slicesData.Marker(allSliceEnable));
         areMarkersKnown = all(ismember(slicesMarkers, string(categories(lineMarkers))));
         if ~areMarkersKnown
            errorMessage = 'Utilized unknown line markers.';
            return;
         end
      end
   end

   function updateGraphs(complexFcn, linLogFcn, EOrH)
      % set automatic Y axis
      hAxSlices.YLimMode = 'auto';
      % convert functions to string to simply check its content
      complexFcnStr = func2str(complexFcn);
      linLogFcnStr = func2str(linLogFcn);
      isLogScale = contains(linLogFcnStr, 'log');
      % set units
      if isLogScale
         units = 'dB';
      else % liner scale
         if contains(complexFcnStr, 'angle')
            hAxSlices.YLim = [-180, 180];
            units = 'deg.';
         elseif EOrH == 'E'
            units = 'V/m';
         elseif EOrH == 'H'
            units = 'A/m';
         end
      end
      % switch between angle and phase
      if contains(complexFcnStr, 'angle')
         complexFcnStr = 'phase';
      end
      % create Y-axis label and graph title
      fieldGraphTitle = sprintf('%s(%s) (%s)', complexFcnStr, EOrH, units);
      hAxField.Title.String = fieldGraphTitle;
      % compute wanted field quatity and show it
      CData = linLogFcn(complexFcn(fieldInPlane));
      % show data as color data, Z-data are zero
      set(hSurf, 'XData', X, 'YData', Y, 'ZData', zeros(size(fieldInPlane, [1, 2])), ...
         'CData', CData);
      % manage linear and dB scale
      if isLogScale
         hSliderDBRange.Enable = true;
         hAxField.CLim = [-maxDBRange, 0];
      else
         hSliderDBRange.Enable = false;
         hAxField.CLimMode = 'auto';
      end
      % delete previous slices lines and fields
      hSlicesLines.delete();
      hSlicesField.delete();
      % alocate new lines
      hSlicesLines = gobjects(nEnSlices, 1);
      hSlicesField = gobjects(nEnSlices, 1);
      dropDownItems = cell(nEnSlices, 1);
      % create new lines
      for iSlice = 1:nEnSlices
         hSlicesLines(iSlice) = line(hAxField, 'XData', slicesX{iSlice}, ...
            'YData', slicesY{iSlice}, 'Marker', slicesMarkers(iSlice), ...
            'MarkerEdgeColor', slicesColors(iSlice), ...
            'Color', slicesColors(iSlice), 'LineWidth', 1);
         
         hSlicesField(iSlice) = line(hAxSlices, 'XData', slicesLength{iSlice}, ...
            'YData', linLogFcn(complexFcn(fieldAtSlices{iSlice})), ...
            'Marker', slicesMarkers(iSlice), ...
            'MarkerEdgeColor', slicesColors(iSlice), ...
            'Color', slicesColors(iSlice), 'LineWidth', 2, 'Visible', false);
         dropDownItems{iSlice} = sprintf('Slice %u', sliceID(iSlice));
      end
      hAxSlices.YAxis.Label.String = fieldGraphTitle;
      set(hDropDownSlicePick, 'Items', dropDownItems, 'ItemsData', (1:nEnSlices).');
      selectSlice(); % check selected value in drop down menu and show relevant slice
      drawnow(); % finish drawing of all graphs and continue
   end

   function selectSlice()
      % show slices according to drop down menu selection
      if nEnSlices
         set(hSlicesField, 'Visible', false); % hide all slicded fields
         set(hSlicesLines, 'LineWidth', 1); % set width of all slices to 1
         % show appropriate sliced field
         hSlicesField(hDropDownSlicePick.Value).Visible = true;
         % make appropriate slice thicker
         hSlicesLines(hDropDownSlicePick.Value).LineWidth = 2;
      end
   end

   function errorMessage = readSources()
      errorMessage = '';
      sourcesData = hTableSources.Data;
      % source powers in wats
      sourcePowers = 10.^(sourcesData.("Power (dBm)")/10)*1e-3;
      try % check power values
         validateattributes(sourcePowers, {'numeric'}, {'column', 'finite'});
      catch ME
         errorMessage = ['Invalid source power. ', ME.message];
         return;
      end
      sourcePhases = sourcesData.("Phase (deg.)")/180*pi; % convert to rad.
      try % check phase values
         validateattributes(sourcePhases, {'numeric'}, {'column', 'finite'});
      catch ME
         errorMessage = ['Invalid source phase. ', ME.message];
         return;
      end
      sourcePositions = arrayfun(@str2num, ...
         sourcesData.("Position (m)"), 'UniformOutput', false);
      % check source positions
      try
         cellfun(@(x)validateattributes(x, {'numeric'}, {'size', [1, 2], 'finite'}), ...
            sourcePositions);
      catch ME
         errorMessage = ['Invalid source position. ', ME.message];
         return;
      end
      sourcesX = cellfun(@(x)x(1), sourcePositions);
      sourcesY = cellfun(@(x)x(2), sourcePositions);
      sourceAzimuths = sourcesData.("Azimuth (deg.)")/180*pi;
      try % check phase values
         validateattributes(sourceAzimuths, {'numeric'}, {'column', 'finite'});
      catch ME
         errorMessage = ['Invalid source azimuth. ', ME.message];
         return;
      end
      % get data just from enabled sources
      sourceEnable = sourcesData.Enable;
      sourcePowers = sourcePowers(sourceEnable, :);
      sourcePhases = sourcePhases(sourceEnable, :);
      sourcesX = sourcesX(sourceEnable, :);
      sourcesY = sourcesY(sourceEnable, :);
      sourceAzimuths = sourceAzimuths(sourceEnable, :);
      % From column vectors create matrices with size [1 x 1 x N], where N is
      % number of sources. Style of computation of field assumes that third
      % dimension belongs to fields from individual sources. Changing size of
      % source data is made by shifting dimensions to the right about 2.
      sourcePowers = shiftdim(sourcePowers, -2);
      sourcePhases = shiftdim(sourcePhases, -2);
      sourcesX = shiftdim(sourcesX, -2);
      sourcesY = shiftdim(sourcesY, -2);
      sourceAzimuths = shiftdim(sourceAzimuths, -2);
   end

   function errorMessage = readSolSpace()
      % read and check size od solution space
      % actual values are checked by edit fields
      errorMessage = '';
      XMin = hEditXMin.Value;
      XMax = hEditXMax.Value;
      YMin = hEditYMin.Value;
      YMax = hEditYMax.Value;
      % check size of solution space
      areLimitsValid = diff([XMin, XMax]) && diff([YMin, YMax]);
      if ~areLimitsValid
         errorMessage = 'Zero solution space.';
         return;
      end
      % check numbers of points as integers
      XSamples = hEditXSamples.Value;
      YSamples = hEditYSamples.Value;
      areSamplesValid = ~mod(XSamples, 1) && ~mod(YSamples, 1);
      if ~areSamplesValid
         errorMessage = 'Wrong number of samples.';
         return;
      end
      % cratecoordinates of all points in solution space
      X = linspace(XMin, XMax, XSamples);
      Y = linspace(YMin, YMax, YSamples).';
      hAxField.XLim = sort([XMin, XMax]);
      hAxField.YLim = sort([YMin, YMax]);
   end

   function changeDynamicRange(eventData)
      % change range of results in dB
      maxDBRange = eventData.Value;
      % zero range is nonsence actually but it is nice to have this option shown on slider
      if maxDBRange < 0.1
         maxDBRange = 0.1;
      end
      % change range of colorbar
      hAxField.CLim = [-maxDBRange, 0];
   end

end